@@ -85,6 +85,11 @@ typedef ngx_js_loc_conf_t *(*ngx_js_external_loc_conf_pt)(njs_external_ptr_t e);
 typedef ngx_js_ctx_t *(*ngx_js_external_ctx_pt)(njs_external_ptr_t e);
 
 
+typedef struct {
+    ngx_array_t          *native_modules;
+} ngx_js_core_conf_t;
+
+
 typedef struct {
     ngx_str_t              name;
     ngx_str_t              path;
@@ -141,7 +146,24 @@ typedef struct {
     ngx_msec_t             fetch_keepalive_time;                              \
     ngx_msec_t             fetch_keepalive_timeout;                           \
     ngx_queue_t            fetch_keepalive_cache;                             \
-    ngx_queue_t            fetch_keepalive_free
+    ngx_queue_t            fetch_keepalive_free;                              \
+                                                                              \
+    ngx_url_t              *fetch_proxy_url;                                  \
+    ngx_str_t               fetch_proxy_auth_header;                          \
+                                                                              \
+    ngx_int_t             (*eval_proxy_url)(ngx_pool_t *pool,                 \
+                                            void *request,                    \
+                                            void *module_conf,                \
+                                            ngx_url_t **url_out,              \
+                                            ngx_str_t *auth_out)
+
+#define ngx_js_conf_dynamic_proxy(conf)                                       \
+     ((conf)->eval_proxy_url != NULL)
+
+#define ngx_js_conf_proxy(conf)                                               \
+    (((conf)->fetch_proxy_url != NULL                                         \
+      && (conf)->fetch_proxy_url->host.len > 0)                               \
+     || ngx_js_conf_dynamic_proxy(conf))
 
 
 #if (NGX_SSL)
@@ -228,6 +250,7 @@ typedef struct ngx_engine_opts_s {
     } u;
 
     njs_str_t                   file;
+    ngx_js_core_conf_t         *core_conf;
     ngx_js_loc_conf_t          *conf;
     ngx_engine_t             *(*clone)(ngx_js_ctx_t *ctx,
                                         ngx_js_loc_conf_t *cf, njs_int_t pr_id,
@@ -275,6 +298,8 @@ struct ngx_engine_s {
     const char                 *name;
     njs_mp_t                   *pool;
     njs_arr_t                  *precompiled;
+    njs_arr_t                  *native_modules;
+    ngx_js_core_conf_t         *core_conf;
 };
 
 
@@ -327,7 +352,7 @@ void ngx_js_ctx_init(ngx_js_ctx_t *ctx, ngx_log_t *log);
 void ngx_js_ctx_destroy(ngx_js_ctx_t *ctx, ngx_js_loc_conf_t *conf);
 ngx_int_t ngx_js_call(njs_vm_t *vm, njs_function_t *func,
     njs_opaque_value_t *args, njs_uint_t nargs);
-ngx_int_t ngx_js_exception(njs_vm_t *vm, ngx_str_t *s);
+ngx_int_t ngx_js_log_exception(njs_vm_t *vm, ngx_log_t *log, const char *txt);
 ngx_engine_t *ngx_njs_clone(ngx_js_ctx_t *ctx, ngx_js_loc_conf_t *cf,
     void *external);
 
@@ -362,7 +387,8 @@ void ngx_engine_qjs_destroy(ngx_engine_t *e, ngx_js_ctx_t *ctx,
     ngx_js_loc_conf_t *conf);
 ngx_int_t ngx_qjs_call(JSContext *cx, JSValue function, JSValue *argv,
     int argc);
-ngx_int_t ngx_qjs_exception(ngx_engine_t *e, ngx_str_t *s);
+ngx_int_t ngx_qjs_log_exception(ngx_engine_t *e, ngx_log_t *log,
+    const char *txt);
 ngx_int_t ngx_qjs_integer(JSContext *cx, JSValueConst val, ngx_int_t *n);
 ngx_int_t ngx_qjs_string(JSContext *cx, ngx_pool_t *pool, JSValueConst val,
     ngx_str_t *dst);
@@ -423,6 +449,9 @@ void ngx_js_logger(ngx_connection_t *c, ngx_uint_t level,
 char * ngx_js_import(ngx_conf_t *cf, ngx_command_t *cmd, void *conf);
 char * ngx_js_engine(ngx_conf_t *cf, ngx_command_t *cmd, void *conf);
 char * ngx_js_preload_object(ngx_conf_t *cf, ngx_command_t *cmd, void *conf);
+char * ngx_js_fetch_proxy(ngx_conf_t *cf, ngx_command_t *cmd, void *conf);
+ngx_int_t ngx_js_parse_proxy_url(ngx_pool_t *pool, ngx_log_t *log,
+    ngx_str_t *url_str, ngx_url_t **url_out, ngx_str_t *auth_header_out);
 ngx_int_t ngx_js_merge_vm(ngx_conf_t *cf, ngx_js_loc_conf_t *conf,
     ngx_js_loc_conf_t *prev,
     ngx_int_t (*init_vm)(ngx_conf_t *cf, ngx_js_loc_conf_t *conf));
@@ -434,6 +463,11 @@ char * ngx_js_merge_conf(ngx_conf_t *cf, void *parent, void *child,
 char *ngx_js_shared_dict_zone(ngx_conf_t *cf, ngx_command_t *cmd, void *conf,
     void *tag);
 
+void *ngx_js_core_create_conf(ngx_cycle_t *cycle);
+char *ngx_js_core_load_native_module(ngx_conf_t *cf, ngx_command_t *cmd,
+    void *conf);
+void ngx_js_native_module_cleanup(void *data);
+
 njs_int_t ngx_js_ext_string(njs_vm_t *vm, njs_object_prop_t *prop, uint32_t unused,
     njs_value_t *value, njs_value_t *setval, njs_value_t *retval);
 njs_int_t ngx_js_ext_uint(njs_vm_t *vm, njs_object_prop_t *prop, uint32_t unused,
