@@ -751,30 +751,17 @@ static const u_char *
 njs_json_parse_number(njs_json_parse_ctx_t *ctx, njs_value_t *value,
     const u_char *p)
 {
-    double        num;
-    njs_int_t     sign;
-    const u_char  *start;
+    double      num;
+    const char  *start, *end;
 
-    sign = 1;
+    start = (const char *) p;
+    num = njs_atod((const char *) p, &end, 10, 0);
 
-    if (*p == '-') {
-        if (p + 1 == ctx->end) {
-            goto error;
-        }
-
-        p++;
-        sign = -1;
+    if (end != start && !isnan(num)) {
+        njs_set_number(value, num);
+        return (const u_char *) end;
     }
 
-    start = p;
-    num = njs_number_dec_parse(&p, ctx->end, 0);
-    if (p != start) {
-        njs_set_number(value, sign * num);
-        return p;
-    }
-
-error:
-
     njs_json_parse_exception(ctx, "Unexpected number", p);
 
     return NULL;
@@ -1803,7 +1790,12 @@ njs_dump_terminal(njs_json_stringify_t *stringify, njs_chb_t *chain,
 
         njs_chb_append_literal(chain, "[");
 
-        njs_typed_array_to_chain(stringify->vm, chain, array, NULL);
+        if (!njs_is_detached_buffer(array->buffer)) {
+            njs_typed_array_to_chain(stringify->vm, chain, array, NULL);
+
+        } else {
+            njs_chb_append_literal(chain, "<detached buffer>");
+        }
 
         njs_chb_append_literal(chain, "]");
 
