@@ -191,13 +191,20 @@ njs_vm_ctor_push(njs_vm_t *vm)
 }
 
 
+njs_int_t
+njs_vm_call_exit_hook(njs_vm_t *vm)
+{
+    if (vm->hooks[NJS_HOOK_EXIT] != NULL) {
+        return njs_vm_call(vm, vm->hooks[NJS_HOOK_EXIT], NULL, 0);
+    }
+
+    return NJS_OK;
+}
+
+
 void
 njs_vm_destroy(njs_vm_t *vm)
 {
-    if (vm->hooks[NJS_HOOK_EXIT] != NULL) {
-        (void) njs_vm_call(vm, vm->hooks[NJS_HOOK_EXIT], NULL, 0);
-    }
-
     njs_mp_destroy(vm->mem_pool);
 }
 
@@ -1513,31 +1520,12 @@ njs_vm_value_to_string(njs_vm_t *vm, njs_str_t *dst, njs_value_t *src)
 }
 
 
-/*
- * If string value is null-terminated the corresponding C string
- * is returned as is, otherwise the new copy is allocated with
- * the terminating zero byte.
- */
 const char *
 njs_vm_value_to_c_string(njs_vm_t *vm, njs_value_t *value)
 {
-    u_char  *p, *data;
-    size_t  size;
-
     njs_assert(njs_is_string(value));
 
-    size = value->string.data->size;
-
-    data = njs_mp_alloc(vm->mem_pool, size + njs_length("\0"));
-    if (njs_slow_path(data == NULL)) {
-        njs_memory_error(vm);
-        return NULL;
-    }
-
-    p = njs_cpymem(data, value->string.data->start, size);
-    *p++ = '\0';
-
-    return (const char *) data;
+    return (const char *) value->string.data->start;
 }
 
 
@@ -1553,7 +1541,8 @@ njs_value_to_string(njs_vm_t *vm, njs_value_t *dst, njs_value_t *value)
             value = njs_object_value(value);
 
         } else {
-            ret = njs_value_to_primitive(vm, &primitive, value, 1);
+            ret = njs_value_to_primitive(vm, &primitive, value,
+                                         NJS_HINT_STRING);
             if (njs_slow_path(ret != NJS_OK)) {
                 return ret;
             }
